#!/usr/bin/env python
#
# Copyright (C) 2005 Holger Mueller <dg5dbh@darc.de>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

import serial
from threading import Thread,Lock
from time import sleep

TTY = "/dev/ttyUSB0"

PTY = ('Undefined', 'News', 'Current Affairs', 'Information', 'Sport',
    'Education', 'Drama', 'Culture', 'Science', 'Varied', 'Pop Music',
    'Rock Music', 'M.O.R Music', 'Light Classical', 'Serious Classical',
    'Other Music', 'Weather', 'Finance', 'Children\'s Program',
    'Social Affairs', 'Religion', 'Phone In', 'Travel', 'Leisure',
    'Jazz Music', 'Country Music', 'National Music', 'Oldies Music',
    'Folk Music', 'Documemtary', 'Alarm Test', 'Alarm')

S_SEARCH, S_COMMAND, S_FREQ = range(3)

class FiFiCom(Thread):

    def __init__(self,device):
	Thread.__init__(self)
	self.device = device
	self.freq = 0
	self.lck = Lock()
	self.ctl = []
	self.state = S_SEARCH
	self._exit = 0

    def run(self):
	while 1:
	    if self._exit: break

	    c = self.device.read(1)

	    if len(c) != 1:	# no input
		sleep(1)
		continue

	    if ord(c) >= 0x80:	# rds stream
		pass
	    else:		# control stream
		self.handleCTL(ord(c))
		
    def handleCTL(self, byte):
	if self.state == S_SEARCH:
	    if byte == 0x1b: self.state = S_COMMAND
	elif self.state == S_COMMAND:
	    if byte == 0x51: self.state = S_FREQ
	    else: self._ctlstate = S_SEARCH
	elif self.state == S_FREQ:
	    self.ctl.append(byte & 0x0f)
	    if len(self.ctl) == 4:
		c = self.ctl
		self.lck.acquire()
		self.freq = (c[0] * 4096 + c[1] * 256 + c[2] * 16 + c[3]) \
		    * 8192 + 22500
		self.lck.release()
		self.ctl = []	
		self.state = S_SEARCH
		
    def setFreq(self, freq):
	s = "\x1b\x51"	# start command sequence
	
	value = (freq - 22500) / 8192
	s += chr(value / 4096)
	value %= 4096
	s += chr(value / 256)
	value %= 256
	s += chr(value / 16)
	value %= 16
	s += chr(value)  

	s += "\x0d"

	self.device.write(s)
	self.device.flush()
	
	self.lck.acquire()
	self.freq = freq
	self.lck.release()

    def getFreq(self):
	self.lck.acquire()
	freq = self.freq
	self.lck.release()
	return freq

    def exit(self):
	self._exit = 1


if __name__ == "__main__":
    TTY = "/dev/ttyUSB0"
    ser = serial.Serial(port=TTY, baudrate=2400, timeout=1)
