#!/usr/bin/env python
#
# Copyright (C) 2005 Holger Mueller <dg5dbh@darc.de>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

import sys
import serial
import fificontrol

from Tkinter import *
import thread
from time import sleep

class RadioDisplay:
    def __init__(self, radio):
	self.radio = radio
	self.window = Tk()
	self.window.title("FiFiRadio")
	self.frequency = Label(self.window, font = ("Arial", 20),
	    text = "%6.2f MHz" % round(radio.getFreq()/1000000., 3)) 
	self.ruler = Scale(self.window, length = 412, from_ = 87500000,
	    to=108000000, resolution = 50000, orient = HORIZONTAL,
	    showvalue = 0, command = self.setFrequency)
	self.frequency.pack(side = TOP)
	self.ruler.pack(side = BOTTOM)
	self.updateThread()
	self.window.mainloop()
	self.radio.exit()
	SystemExit

    def setFrequency(self, event):
	freq = int(self.ruler.get())
	self.frequency.config(text = "%6.2f MHz" % round(freq / 1000000., 3))
	self.radio.setFreq(freq)

    def updateThread(self):
	thread.start_new_thread(self.update,())

    def update(self):
	oldfreq = 87500000
	while True:
	    sleep(1)
	    freq = self.radio.getFreq()
	    if freq != oldfreq: 
		self.frequency.config(text = "%6.2f MHz" %
		    round(freq / 1000000., 3))
		self.ruler.set(freq)
		oldfreq = freq
	    
if __name__ == "__main__":
    if len(sys.argv) > 1:
	tty = sys.argv[1]
    else:
	tty = "/dev/ttyUSB0"
    ser = serial.Serial(port=tty, baudrate=2400, timeout=1)
    radio = fificontrol.FiFiCom(ser)
    radio.start()
    display = RadioDisplay(radio)
